﻿using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;
using System.Web;

namespace SYS_Core
{
    internal class TokenValidateHandler : DelegatingHandler
    {
        private string jwtKey = ConfigurationManager.AppSettings["JWT_KEY"];
        private static bool TryRetrieveToken(HttpRequestMessage request, out string token)
        {
            token = null;
            IEnumerable<string> authzHeaders;
            if (!request.Headers.TryGetValues("Authorization", out authzHeaders) || authzHeaders.Count() > 1)
            {
                return false;
            }
            var bearerToken = authzHeaders.ElementAt(0);
            token = bearerToken.StartsWith("Bearer ") ? bearerToken.Substring(7) : bearerToken;
            return true;
        }

        protected override async Task<HttpResponseMessage> SendAsync(HttpRequestMessage request, CancellationToken cancellationToken)
        {
            HttpStatusCode statusCode;
            var response = new HttpResponseMessage();
            string token;
            //determine whether a jwt exists or not
            if (!TryRetrieveToken(request, out token))
            {
                statusCode = HttpStatusCode.Unauthorized;
                //allow requests with no token - whether a action method needs an authentication can be set with the claimsauthorization attribute
                return await base.SendAsync(request, cancellationToken);
            }

            try
            {
                var now = DateTime.UtcNow;
                var securityKey = new Microsoft.IdentityModel.Tokens.SymmetricSecurityKey(System.Text.Encoding.UTF8.GetBytes(jwtKey));
                Boolean isValidateLifeTime = true;


                SecurityToken securityToken;
                JwtSecurityTokenHandler handler = new JwtSecurityTokenHandler();
                //extract and assign the user of the jwt
                System.Security.Claims.ClaimsPrincipal principal;
                try
                {
                    isValidateLifeTime = true;
                    TokenValidationParameters validationParameters = new TokenValidationParameters()
                    {
                        ValidAudience = "NMT",
                        ValidIssuer = "NMT",
                        ValidateLifetime = true,
                        ValidateIssuerSigningKey = true,
                        LifetimeValidator = this.LifetimeValidator,
                        IssuerSigningKey = securityKey
                    };

                    principal = handler.ValidateToken(token, validationParameters, out securityToken);

                } catch (Exception ex)
                {
                    isValidateLifeTime = false;
                    TokenValidationParameters validationParameters = new TokenValidationParameters()
                    {
                        ValidAudience = "NMT",
                        ValidIssuer = "NMT",
                        ValidateLifetime = false,
                        ValidateIssuerSigningKey = true,
                        IssuerSigningKey = securityKey
                    };

                    principal = handler.ValidateToken(token, validationParameters, out securityToken);
                }

                var identity = principal.Identity as System.Security.Claims.ClaimsIdentity;
                var userName = identity.Claims.FirstOrDefault(x => x.Type == aControl.Cons.SS_USER_NAME).Value;



                //Verify username
                if (userName != null && (isValidateLifeTime || (!isValidateLifeTime && userName.Substring(0,3) == "LL_")))
                {
                    if (userName.Substring(0, 3) == "LL_")
                    {
                        aControl.DBManager odb = new aControl.DBManager();

                        if(!odb.CheckToken(userName, token, null)) {
                            throw new Exception("Unknown authorization identity.");
                        }
                    }

                    Thread.CurrentPrincipal = principal;
                    HttpContext.Current.User = principal;

                    response = await base.SendAsync(request, cancellationToken);
                } else
                {
                    throw new Exception("Unknown authorization identity.");
                }

            }
            catch (SecurityTokenValidationException e)
            {
                //statusCode = HttpStatusCode.Unauthorized;

                response.StatusCode = HttpStatusCode.Unauthorized;
                response = await base.SendAsync(request, cancellationToken);
            }
            catch (Exception ex)
            {
                //statusCode = HttpStatusCode.InternalServerError;

                response.StatusCode = HttpStatusCode.Unauthorized;
                response = await base.SendAsync(request, cancellationToken);
            }

            return response;
            //return Task<HttpResponseMessage>.Factory.StartNew(() => new HttpResponseMessage(statusCode) { });
        }

        public bool LifetimeValidator(DateTime? notBefore, DateTime? expires, SecurityToken securityToken, TokenValidationParameters validationParameters)
        {
            if (expires != null)
            {
                if (DateTime.UtcNow < expires) return true;
            }
            return false;
        }
    }
}