﻿using Microsoft.IdentityModel.Tokens;
using SYS_Core.aControl;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Security.Cryptography;
using System.Text;
using System.Web;

namespace SYS_Core.Services
{
    public class TokenService : ITokenService
    {
        private string jwtKey;
        private SymmetricSecurityKey secretKey;
        public TokenService()
        {
            jwtKey = ConfigurationManager.AppSettings["JWT_KEY"];
            secretKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(jwtKey));
        }
        public string GenerateAccessToken(IEnumerable<Claim> claims)
        {
            var signinCredentials = new SigningCredentials(secretKey, SecurityAlgorithms.HmacSha256);

            var tokeOptions = new JwtSecurityToken(
                issuer: "NMT",
                audience: "NMT",
                claims: claims,
                expires: DateTime.UtcNow.AddMinutes(int.Parse(ConfigurationManager.AppSettings["TOKEN_DURATION"])),
                signingCredentials: signinCredentials
            );

            var tokenString = new JwtSecurityTokenHandler().WriteToken(tokeOptions);
            AssignClaims(tokenString);

            return tokenString;
        }

        public string GenerateAccessTokenHandheld(IEnumerable<Claim> claims)
        {
            var signinCredentials = new SigningCredentials(secretKey, SecurityAlgorithms.HmacSha256);

            var tokeOptions = new JwtSecurityToken(
                issuer: "NMT",
                audience: "NMT",
                claims: claims,
                expires: DateTime.UtcNow.AddMinutes(int.Parse(ConfigurationManager.AppSettings["TOKEN_DURATION_HH"])),
                signingCredentials: signinCredentials
            );

            var tokenString = new JwtSecurityTokenHandler().WriteToken(tokeOptions);
            AssignClaims(tokenString);

            return tokenString;
        }

        public string GenerateAccessTokenNoExpiration(IEnumerable<Claim> claims)
        {
            var signinCredentials = new SigningCredentials(secretKey, SecurityAlgorithms.HmacSha256);

            var tokeOptions = new JwtSecurityToken(
                issuer: "NMT",
                audience: "NMT",
                claims: claims,
                signingCredentials: signinCredentials
            );

            var tokenString = new JwtSecurityTokenHandler().WriteToken(tokeOptions);
            AssignClaims(tokenString);

            return tokenString;
        }

        public string GenerateQRToken(string pJwtKey, JwtPayload pPayload)
        {
            var lSecretKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(pJwtKey));
            var signinCredentials = new SigningCredentials(lSecretKey, SecurityAlgorithms.HmacSha256);

            var header = new JwtHeader(signinCredentials);

            var tokeOptions = new JwtSecurityToken(header, pPayload);

            var tokenString = new JwtSecurityTokenHandler().WriteToken(tokeOptions);

            return tokenString;
        }

        public string GenerateRefreshToken()
        {
            var randomNumber = new byte[32];
            using (var rng = RandomNumberGenerator.Create())
            {
                rng.GetBytes(randomNumber);

                return Convert.ToBase64String(randomNumber);
            }
        }

        public ClaimsPrincipal GetPrincipalFromExpiredToken(string token)
        {
            if (token != null && token != "")
            {
                var tokenValidationParameters = new TokenValidationParameters
                {
                    ValidAudience = "NMT",
                    ValidIssuer = "NMT",
                    ValidateAudience = false,
                    ValidateIssuer = false,
                    ValidateIssuerSigningKey = true,
                    IssuerSigningKey = secretKey,
                    ValidateLifetime = false
                };

                var tokenHandler = new JwtSecurityTokenHandler();
                SecurityToken securityToken;
                var principal = tokenHandler.ValidateToken(token, tokenValidationParameters, out securityToken);
                var jwtSecurityToken = securityToken as JwtSecurityToken;
                if (jwtSecurityToken == null || !jwtSecurityToken.Header.Alg.Equals(SecurityAlgorithms.HmacSha256, StringComparison.InvariantCultureIgnoreCase))
                    throw new SecurityTokenException("Invalid token");

                return principal;
            }

            return null;
        }

        private void AssignClaims(string token)
        {
            //extract and assign the user of the jwt
            var handlerToken = GetPrincipalFromExpiredToken(token);

            //Assign token object
            System.Threading.Thread.CurrentPrincipal = handlerToken;
            HttpContext.Current.User = handlerToken;
        }
    }
}