﻿using System;
using System.Collections.Generic;
using Quartz;
using Quartz.Impl;
using System.IO;
using System.Globalization;
using System.Threading.Tasks;

namespace SYS_Core.aBiz
{
    public static class JobSchedule
    {

 
        public static async Task CreateJob(string jobName, string groupName,string cronText)
        {
 

            //// Delete before update
            await DeleteJob(jobName, groupName);

            //// Create Schedule
            IScheduler scheduler = await StdSchedulerFactory.GetDefaultScheduler();

            //// Create Job
            IJobDetail job = JobBuilder.Create<JobTask>()
                .WithIdentity(jobName, groupName)
                .UsingJobData("JobName", jobName)
                .Build();

            //// Create Trigger
            ITrigger trigger;
            string triggerName = Guid.NewGuid().ToString().ToLower();
           // string cronText = String.Format("0 {0} {1} {2}/{3} * ? *", model.StartDateTime.Minute, model.StartDateTime.Hour, model.StartDateTime.Day, model.FrequencyDay);
            trigger = TriggerBuilder.Create()
                .WithIdentity(triggerName, groupName)
                .StartNow()
                .WithCronSchedule(cronText, x => x.InTimeZone(TimeZoneInfo.FindSystemTimeZoneById("SE Asia Standard Time")))
                .Build();

            //// Add Job & Trigger to Schedule
            await scheduler.ScheduleJob(job, trigger);
            await scheduler.Start();
        }

        public static async Task DeleteJob(string jobName, string groupName = "PMS")
        {
            ISchedulerFactory sf = new StdSchedulerFactory();
            IScheduler sched = await sf.GetScheduler();
            await sched.DeleteJob(new JobKey(jobName, groupName));
        }

    }

    public class JobTask : IJob
    {
        public async Task Execute(IJobExecutionContext context)
        {
            string folderPath = System.AppDomain.CurrentDomain.BaseDirectory + "Log";
            if (!Directory.Exists(folderPath))
            {
                Directory.CreateDirectory(folderPath);
            }
            string fileName = "ScheduleLog_" + DateTime.Now.ToString("yyyy-MM-dd", CultureInfo.InvariantCulture) + ".txt";
            string filePath = Path.Combine(folderPath, fileName);
            string str = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss", CultureInfo.InvariantCulture) + " - Started Job";
            //File.AppendAllLines(filePath, new string[] { str });

            try
            {
                JobDataMap dataMap = context.JobDetail.JobDataMap;
                string JobName = dataMap.Values.ToString();
                JobService.StartService(JobName);
            }
            catch (Exception ex)
            {
                str = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss", CultureInfo.InvariantCulture) + " - Error : " + ex.Message;
                File.AppendAllLines(filePath, new string[] { str });
            }

            //str = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss", CultureInfo.InvariantCulture) + " - Finished Job" + Environment.NewLine + Environment.NewLine;
            //File.AppendAllLines(filePath, new string[] { str });
        }
    }

    public class JobItem
    {
        public string jobId { get; set; }
        public IList<ITrigger> Triggers { get; set; }
    }
}