﻿using Microsoft.Win32;
using SYS_Core.aControl;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Printing;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Security.Principal;
using System.Threading;
using System.Threading.Tasks;
using System.Web;
using static SYSCore.aControl.EmailSender;

namespace SYSCore.aControl
{
    public class PrintService
    {
        private byte[][] m_renderedReport;
        private Graphics.EnumerateMetafileProc m_delegate = null;
        private MemoryStream m_currentPageStream;
        private Metafile m_metafile = null;
        private int m_numberOfPages;
        private int m_currentPrintingPage;
        private int m_lastPrintingPage;

        public bool PrintReport(ReportImageResult reportResult, string printerName, int paperWidth, int paperHeight)
        {
            this.m_renderedReport = reportResult.pages;
            this.m_numberOfPages = reportResult.numberOfPages;

            try
            {
                //You can set the Required Printer Settings(Paper Size, Page Source, Orientation etc) in printerSettings object defined below
                PrinterSettings printerSettings = new PrinterSettings();
                printerSettings.MaximumPage = m_numberOfPages;
                printerSettings.MinimumPage = 1;
                printerSettings.PrintRange = PrintRange.AllPages;//PrintRange.SomePages;

                PrintDocument pd = new PrintDocument();

                m_currentPrintingPage = 1;
                m_lastPrintingPage = m_numberOfPages;

                Console.WriteLine("Printing report...");

                using (WindowsImpersonationContext wic = WindowsIdentity.Impersonate(IntPtr.Zero))
                {
                    //code to send printdocument to the printer
                    //Set Your Printer Name here
                    printerSettings.PrinterName = printerName;
                    printerSettings.DefaultPageSettings.PaperSize = new PaperSize("CustomPaperSize", paperWidth, paperHeight);
                    pd.PrinterSettings = printerSettings;
                    pd.PrintPage += this.pd_PrintPage;

                    pd.Print();
                }
            }
            catch (Exception ex)
            {
                //System.Windows.Forms.MessageBox.Show(ex.ToString());
                Console.WriteLine(ex.Message);
            }
            return true;
        }

        //public async Task<bool> CreatePrintRecord(string fileLocation, string printerName = null)
        //{
        //    try
        //    {
        //        await Task.Run(() =>
        //        {
        //            string applicationPath = FindApplicationPath();
        //            string printArguments = String.Format(@"/h /p {0}", fileLocation);

        //            if (printerName != null)
        //            {
        //                printArguments = String.Format(@"/h /t ""{0}"" ""{1}""", fileLocation, printerName);
        //            }

        //            Process proc = new Process();
        //            proc.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
        //            proc.StartInfo.Verb = "print";

        //            //Define location of adobe reader/command line
        //            //switches to launch adobe in "print" mode
        //            //proc.StartInfo.FileName = applicationPath;
        //            //proc.StartInfo.Arguments = printArguments;
        //            proc.StartInfo.FileName = fileLocation;

        //            proc.StartInfo.UseShellExecute = false;
        //            proc.StartInfo.CreateNoWindow = true;

        //            proc.Start();
        //            proc.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
        //            if (proc.HasExited == false)
        //            {
        //                proc.WaitForExit(10000);
        //            }

        //            proc.EnableRaisingEvents = true;

        //            proc.Close();
        //            //KillAdobe("AcroRd32");
        //            return true;

        //        });

        //        return true;
        //    }
        //    catch (Exception ex)
        //    {
        //        return false;
        //    }
        //}

        //private string FindApplicationPath()
        //{
        //    string applicationPath = "";

        //    var printApplicationRegistryPaths = new[]
        //    {
        //        @"SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\Acrobat.exe",
        //        @"SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\AcroRD32.exe"
        //    };

        //    foreach (var printApplicationRegistryPath in printApplicationRegistryPaths)
        //    {
        //        using (var regKeyAppRoot = Registry.LocalMachine.OpenSubKey(printApplicationRegistryPath))
        //        {
        //            if (regKeyAppRoot == null)
        //            {
        //                continue;
        //            }

        //            applicationPath = (string)regKeyAppRoot.GetValue(null);

        //            if (!string.IsNullOrEmpty(applicationPath))
        //            {
        //                return applicationPath;
        //            }
        //        }
        //    }

        //    return applicationPath;
        //}

        //private static bool KillAdobe(string name)
        //{
        //    foreach (Process clsProcess in Process.GetProcesses().Where(
        //                 clsProcess => clsProcess.ProcessName.StartsWith(name)))
        //    {
        //        clsProcess.Kill();
        //        return true;
        //    }
        //    return false;
        //}

        //private bool CheckPrinterStatus(string printerName = null)
        //{
        //    LocalPrintServer printServer = new LocalPrintServer();
        //    PrintQueue printQueue = printerName == null ? printServer.DefaultPrintQueue : printServer.GetPrintQueue(printerName);

        //    if (printQueue.IsOffline)
        //        //Send printer offline
        //        return EmailPrinterFail("PrinterOffline", printerName);

        //    if (printQueue.IsPaperJammed)
        //        //Send paperjammed
        //        return EmailPrinterFail("PrinterPaperJammed", printerName);

        //    if (printQueue.IsOutOfPaper)
        //        //Send printer out of paper
        //        return EmailPrinterFail("PrinterOutOfPaper", printerName);

        //    return true;
        //}

        private bool EmailPrinterFail(string emailConfig, string printerName = null)
        {
            DataService dataService = new DataService();
            EmailConfigure configure = dataService.GetEmailConfigure(emailConfig);
            configure.subject = printerName != null ? String.Concat(configure.subject, $" ({printerName})") : configure.subject;

            //Send email with attachments
            EmailSender sender = new EmailSender();
            sender.Email(configure);

            return false;
        }

        /// <summary>
        /// Handle the Printing of each page
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="ev"></param>
        private void pd_PrintPage(object sender, PrintPageEventArgs ev)
        {
            ev.HasMorePages = false;
            if (m_currentPrintingPage <= m_lastPrintingPage && MoveToPage(m_currentPrintingPage))
            {
                // Draw the page
                ReportDrawPage(ev.Graphics);
                // If the next page is less than or equal to the last page,
                // print another page.
                if (System.Threading.Interlocked.Increment(ref m_currentPrintingPage) <= m_lastPrintingPage)
                {
                    ev.HasMorePages = true;
                }
            }
        }

        // Method to draw the current emf memory stream
        private void ReportDrawPage(Graphics g)
        {
            if (m_currentPageStream == null || 0 == m_currentPageStream.Length || m_metafile == null)
            {
                return;
            }
            lock (this)
            {
                // Set the metafile delegate.
                int width = m_metafile.Width;

                int height = m_metafile.Height;
                m_delegate = new Graphics.EnumerateMetafileProc(MetafileCallback);
                // Draw in the rectangle
                //Point destPoint = new Point(0, 0);
                Point[] points = new Point[3];
                Point destPoint = new Point(0, 0);
                Point destPoint1 = new Point(width / 2, 0);
                Point destPoint2 = new Point(0, height / 2);

                points[0] = destPoint;
                points[1] = destPoint1;
                points[2] = destPoint2;
                g.EnumerateMetafile(m_metafile, points, m_delegate);
                // Clean up
                m_delegate = null;
            }
        }



        private bool MoveToPage(int page)
        {
            // Check to make sure that the current page exists in
            // the array list
            if (m_renderedReport[m_currentPrintingPage - 1] == null)
            {
                return false;
            }
            // Set current page stream equal to the rendered page
            m_currentPageStream = new MemoryStream(m_renderedReport[m_currentPrintingPage - 1]);
            // Set its postion to start.
            m_currentPageStream.Position = 0;
            // Initialize the metafile
            if (m_metafile != null)
            {
                m_metafile.Dispose();
                m_metafile = null;
            }
            // Load the metafile image for this page
            m_metafile = new Metafile((Stream)m_currentPageStream);
            return true;
        }

        private bool MetafileCallback(EmfPlusRecordType recordType, int flags, int dataSize, IntPtr data, PlayRecordCallback callbackData)
        {
            byte[] dataArray = null;
            // Dance around unmanaged code.
            if (data != IntPtr.Zero)
            {
                // Copy the unmanaged record to a managed byte buffer
                // that can be used by PlayRecord.
                dataArray = new byte[dataSize];
                Marshal.Copy(data, dataArray, 0, dataSize);
            }
            // play the record.
            m_metafile.PlayRecord(recordType, flags, dataSize, dataArray);
            return true;
        }
    }
}